"""
A standalone test runner script, configuring the minimum settings
required for tests to execute.

Re-use at your own risk: many Django applications will require
different settings and/or templates to run their tests.

"""

import os
import sys


APP_DIR = os.path.abspath(os.path.dirname(__file__))


# Minimum settings required for the app's tests.
SETTINGS_DICT = {
    "BASE_DIR": APP_DIR,
    "INSTALLED_APPS": (
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.sites",
        "flashpolicies",
    ),
    # Test cases will override this liberally.
    "ROOT_URLCONF": "tests.urls",
    "DATABASES": {
        "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}
    },
    "MIDDLEWARE": (
        "django.middleware.common.CommonMiddleware",
        "django.middleware.csrf.CsrfViewMiddleware",
        "django.contrib.sessions.middleware.SessionMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
    ),
    "SITE_ID": 1,
    "TEMPLATES": [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "DIRS": [os.path.join(APP_DIR, "tests/templates")],
            "OPTIONS": {
                "context_processors": [
                    "django.contrib.auth.context_processors.auth",
                    "django.template.context_processors.debug",
                    "django.template.context_processors.i18n",
                    "django.template.context_processors.media",
                    "django.template.context_processors.static",
                    "django.template.context_processors.tz",
                    "django.contrib.messages.context_processors.messages",
                ]
            },
        }
    ],
}


def run_tests():
    # Making Django run this way is a two-step process. First, call
    # settings.configure() to give Django settings to work with:
    from django.conf import settings

    settings.configure(**SETTINGS_DICT)

    # Then, call django.setup() to initialize the application cache
    # and other bits:
    import django

    django.setup()

    # Now we instantiate a test runner...
    from django.test.utils import get_runner

    TestRunner = get_runner(settings)

    # And then we run tests and return the results.
    test_runner = TestRunner(verbosity=2, interactive=True)
    failures = test_runner.run_tests(["tests"])
    sys.exit(bool(failures))


if __name__ == "__main__":
    run_tests()



import xml.dom.minidom

from django.test import SimpleTestCase

from flashpolicies import policies


class PolicyViewTests(SimpleTestCase):
    """
    Tests the views which serve policy files.

    """

    urls = "flashpolicies.tests.urls"

    def test_serve_response(self):
        """
        Tests that the serve() view returns a byte string response with
        the correct content type and charset.

        """
        response = self.client.get("/crossdomain-serve.xml")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(bytes, type(response.content))
        self.assertEqual(
            response["Content-Type"], "text/x-cross-domain-policy; charset=utf-8"
        )

    def test_serve(self):
        """
        Tests that the serve() view serializes the policy as expected.

        """
        response = self.client.get("/crossdomain-serve.xml")

        # Parse the returned policy and make sure it matches what was
        # passed in.
        policy = xml.dom.minidom.parseString(response.content)
        self.assertEqual(len(policy.getElementsByTagName("allow-access-from")), 1)
        self.assertEqual(
            len(policy.getElementsByTagName("allow-http-request-headers-from")), 1
        )

    def test_allow_domains(self):
        """
        Tests the allow_domains() view.

        """
        response = self.client.get("/crossdomain-allow-domains.xml")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response["Content-Type"], "text/x-cross-domain-policy; charset=utf-8"
        )
        policy = xml.dom.minidom.parseString(response.content)
        self.assertEqual(len(policy.getElementsByTagName("allow-access-from")), 2)
        domains = ["media.example.com", "api.example.com"]
        domains_in_xml = [
            elem.getAttribute("domain")
            for elem in policy.getElementsByTagName("allow-access-from")
        ]
        for domain in domains_in_xml:
            domains.remove(domain)

    def test_allow_domains_alias(self):
        """
        Tests the alias for the allow_domains() view.

        """
        response = self.client.get("/crossdomain-simple-alias.xml")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response["Content-Type"], "text/x-cross-domain-policy; charset=utf-8"
        )
        policy = xml.dom.minidom.parseString(response.content)
        self.assertEqual(len(policy.getElementsByTagName("allow-access-from")), 2)
        domains = ["media.example.com", "api.example.com"]
        domains_in_xml = [
            elem.getAttribute("domain")
            for elem in policy.getElementsByTagName("allow-access-from")
        ]
        for domain in domains_in_xml:
            domains.remove(domain)

    def test_no_access(self):
        """
        Tests the view which generates a policy that forbids all
        access.

        """
        response = self.client.get("/crossdomain-no-access.xml")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response["Content-Type"], "text/x-cross-domain-policy; charset=utf-8"
        )
        policy = xml.dom.minidom.parseString(response.content)
        self.assertEqual(len(policy.getElementsByTagName("site-control")), 1)
        self.assertEqual(
            policy.getElementsByTagName("site-control")[0].getAttribute(
                "permitted-cross-domain-policies"
            ),
            policies.SITE_CONTROL_NONE,
        )

    def test_metapolicy(self):
        """
        Tests the view which sets a meta-policy for allowing other
        policies on the same domain.

        """
        response = self.client.get("/crossdomain-metapolicy.xml")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response["Content-Type"], "text/x-cross-domain-policy; charset=utf-8"
        )
        policy = xml.dom.minidom.parseString(response.content)
        self.assertEqual(len(policy.getElementsByTagName("site-control")), 1)
        self.assertEqual(
            policy.getElementsByTagName("site-control")[0].getAttribute(
                "permitted-cross-domain-policies"
            ),
            policies.SITE_CONTROL_ALL,
        )



"""
Utilities for generating Flash cross-domain policy files.

"""

import xml.dom
from typing import Dict, Iterable, List, Optional


minidom = xml.dom.getDOMImplementation("minidom")


METAPOLICY_ERROR = (
    "Metapolicy currently forbids all access; to {}, change the metapolicy."
)
SITE_CONTROL_ERROR = (
    "'{}' is not a valid value for the 'permitted-cross-domain-policies' "
    "attribute of a 'site-control' element."
)
BAD_POLICY = (
    "Cannot produce XML from invalid policy (metapolicy forbids all access, "
    "but policy attempted to allow access anyway)."
)


#
# Acceptable values for the "permitted-cross-domain-policies"
# attribute of "site-control" elements. See section 3(b)(i) of the
# Adobe crossdomain.xml spec.
#

# All policy files are allowed:
SITE_CONTROL_ALL = "all"

# Only files served as text/x-cross-domain-policy are allowed:
SITE_CONTROL_BY_CONTENT_TYPE = "by-content-type"

# Only files named 'crossdomain.xml' are allowed:
SITE_CONTROL_BY_FTP_FILENAME = "by-ftp-filename"

# Only the master policy file is allowed:
SITE_CONTROL_MASTER_ONLY = "master-only"

# No policies are allowed, including the master policy:
SITE_CONTROL_NONE = "none"

VALID_SITE_CONTROL = (
    SITE_CONTROL_ALL,
    SITE_CONTROL_BY_CONTENT_TYPE,
    SITE_CONTROL_BY_FTP_FILENAME,
    SITE_CONTROL_MASTER_ONLY,
    SITE_CONTROL_NONE,
)


class Policy:
    """
    Wrapper object for creating and manipulating a Flash cross-domain
    policy.

    In the simplest case -- specifying one or more domains to allow
    access from -- simply pass the domains when initializing. For
    example::

        my_policy = Policy('media.example.com', 'api.example.com')

    The property ``xml_dom`` of the returned ``Policy`` object will be
    an ``xml.dom.minidom.Document`` representing the resulting policy,
    and can be serialized for serving via HTTP or writing to a
    file. For ease of use, ``__str__()`` is defined here and returns
    the policy as a Unicode string for easy inspection;
    ``serialize()`` will serialize to UTF-8 for serving or writing to
    a file.

    Consult the documentation for the various methods of this class
    for more advanced uses.

    """

    def __init__(self, *domains: str):
        self.site_control = None  # type: Optional[str]
        self.domains = {}  # type: Dict[str, dict]
        self.header_domains = {}  # type: Dict[str, dict]
        self.identities = []  # type: List[str]
        for domain in domains:
            self.allow_domain(domain)

    def allow_domain(
        self, domain: str, to_ports: Optional[Iterable[str]] = None, secure: bool = True
    ):
        """
        Allows access from ``domain``, which may be either a full
        domain name, or a wildcard (e.g., ``*.example.com``, or simply
        ``*``). Due to security concerns, it is strongly recommended
        that you use explicit domains rather than wildcards.

        For socket policy files, pass a list of ports or port ranges
        as the keyword argument ``to_ports``. As with ``domain``, a
        wildcard value -- ``*`` -- will allow all ports.

        To disable Flash's requirement of security matching (e.g.,
        retrieving a policy via HTTPS will require that SWFs also be
        retrieved via HTTPS), pass ``secure=False``. Due to security
        concerns, it is strongly recommended that you not disable
        this.

        """
        if self.site_control == SITE_CONTROL_NONE:
            raise TypeError(METAPOLICY_ERROR.format("allow a domain"))
        self.domains[domain] = {"to_ports": to_ports, "secure": secure}

    def metapolicy(self, permitted: str):
        """
        Sets metapolicy to ``permitted``. (only applicable to master
        policy files). Acceptable values correspond to those listed in
        Section 3(b)(i) of the crossdomain.xml specification, and are
        also available as a set of constants defined in this module.

        By default, Flash assumes a value of ``master-only`` for all
        policies except socket policies, (which assume a default of
        ``all``) so if this is desired (and, for security, it
        typically is), this method does not need to be called.

        Note that a metapolicy of ``none`` forbids **all** access,
        even if one or more domains, headers or identities have
        previously been specified as allowed. As such, setting the
        metapolicy to ``none`` will remove all access previously
        granted by ``allow_domain``, ``allow_headers`` or
        ``allow_identity``. Additionally, attempting to grant access
        via ``allow_domain``, ``allow_headers`` or ``allow_identity``
        will, when the metapolicy is ``none``, raise ``TypeError``.

        """
        if permitted not in VALID_SITE_CONTROL:
            raise TypeError(SITE_CONTROL_ERROR.format(permitted))
        if permitted == SITE_CONTROL_NONE:
            # Metapolicy 'none' means no access is permitted.
            self.domains = {}
            self.header_domains = {}
            self.identities = []
        self.site_control = permitted

    def allow_headers(self, domain: str, headers: Iterable[str], secure: bool = True):
        """
        Allows ``domain`` to push data via the HTTP headers named in
        ``headers``.

        As with ``allow_domain``, ``domain`` may be either a full
        domain name or a wildcard. Again, use of wildcards is
        discouraged for security reasons.

        The value for ``headers`` should be a list of header names.

        To disable Flash's requirement of security matching (e.g.,
        retrieving a policy via HTTPS will require that SWFs also be
        retrieved via HTTPS), pass ``secure=False``. Due to security
        concerns, it is strongly recommended that you not disable
        this.

        """
        if self.site_control == SITE_CONTROL_NONE:
            raise TypeError(METAPOLICY_ERROR.format("allow headers from a domain"))
        self.header_domains[domain] = {"headers": headers, "secure": secure}

    def allow_identity(self, fingerprint: str):
        """
        Allows access from documents digitally signed by the key with
        ``fingerprint``.

        In theory, multiple algorithms can be added in the future for
        calculating ``fingerprint`` from the signing key, but at this
        time only one algorithm -- SHA-1 -- is supported by the
        cross-domain policy specification.

        """
        if self.site_control == SITE_CONTROL_NONE:
            raise TypeError(
                METAPOLICY_ERROR.format("allow access from signed documents")
            )
        if fingerprint not in self.identities:
            self.identities.append(fingerprint)

    def _add_domains_xml(self, document: xml.dom.minidom.Document):
        """
        Generates the XML elements for allowed domains.

        """
        for domain, attrs in self.domains.items():
            domain_element = document.createElement("allow-access-from")
            domain_element.setAttribute("domain", domain)
            if attrs["to_ports"] is not None:
                domain_element.setAttribute("to-ports", ",".join(attrs["to_ports"]))
            if not attrs["secure"]:
                domain_element.setAttribute("secure", "false")
            document.documentElement.appendChild(domain_element)

    def _add_header_domains_xml(self, document: xml.dom.minidom.Document):
        """
        Generates the XML elements for allowed header domains.

        """
        for domain, attrs in self.header_domains.items():
            header_element = document.createElement("allow-http-request-headers-from")
            header_element.setAttribute("domain", domain)
            header_element.setAttribute("headers", ",".join(attrs["headers"]))
            if not attrs["secure"]:
                header_element.setAttribute("secure", "false")
            document.documentElement.appendChild(header_element)

    def _add_identities_xml(self, document: xml.dom.minidom.Document):
        """
        Generates the XML elements for allowed digital signatures.

        """
        for fingerprint in self.identities:
            identity_element = document.createElement("allow-access-from-identity")
            signatory_element = document.createElement("signatory")
            certificate_element = document.createElement("certificate")
            certificate_element.setAttribute("fingerprint", fingerprint)
            certificate_element.setAttribute("fingerprint-algorithm", "sha-1")
            signatory_element.appendChild(certificate_element)
            identity_element.appendChild(signatory_element)
            document.documentElement.appendChild(identity_element)

    def _get_xml_dom(self) -> xml.dom.minidom.Document:
        """
        Collects all options set so far, and produce and return an
        ``xml.dom.minidom.Document`` representing the corresponding
        XML.

        """
        if self.site_control == SITE_CONTROL_NONE and any(
            (self.domains, self.header_domains, self.identities)
        ):
            raise TypeError(BAD_POLICY)

        policy_type = minidom.createDocumentType(
            qualifiedName="cross-domain-policy",
            publicId=None,
            systemId="http://www.adobe.com/xml/dtds/cross-domain-policy.dtd",
        )
        policy = minidom.createDocument(None, "cross-domain-policy", policy_type)

        if self.site_control is not None:
            control_element = policy.createElement("site-control")
            control_element.setAttribute(
                "permitted-cross-domain-policies", self.site_control
            )
            policy.documentElement.appendChild(control_element)

        for elem_type in ("domains", "header_domains", "identities"):
            getattr(self, "_add_{}_xml".format(elem_type))(policy)

        return policy

    xml_dom = property(_get_xml_dom)

    def __str__(self) -> str:
        return self.xml_dom.toprettyxml()

    def serialize(self) -> bytes:
        """
        Serializes this policy to a UTF-8 byte sequence.

        This is similar to __str__() but with one important
        difference: __str__() is required to return a Unicode string,
        and so can't use the 'encoding' argument of
        toprettyxml(). This method can return a bytes object, and will
        return a UTF-8-encoded byte sequence with appropriate encoding
        declaration in its XML prolog..

        In general, use str() if you just want to see what would be
        produced, and use serialize() if you want to pass the result
        to something that will serve the XML, or write to a file.

        """
        return self.xml_dom.toprettyxml(encoding="utf-8")



"""
Views for generating and serving policy files.

"""

import warnings
from typing import Iterable, Optional

from django.http import HttpRequest, HttpResponse

from . import policies


def serve(request: HttpRequest, policy: policies.Policy) -> HttpResponse:
    """
    Given a ``flashpolicies.policies.Policy`` instance, serializes it
    to XML and serve it.

    Internally, this is used by all other views as the mechanism which
    actually serves the policy file.

    **Required arguments:**

    ``policy``
        The ``flashpolicies.policies.Policy`` instance to serve.

    **Optional arguments:**

    None.

    """
    return HttpResponse(
        policy.serialize(), content_type="text/x-cross-domain-policy; charset=utf-8"
    )


def allow_domains(request: HttpRequest, domains: Iterable[str]) -> HttpResponse:
    """
    Serves a cross-domain access policy allowing a list of domains.

    Note that if this is returned from the URL ``/crossdomain.xml`` on
    a domain, it will act as a master policy and will not permit other
    policies to exist on that domain. If you need to set meta-policy
    information and allow other policies, use the view
    :view:`flashpolicies.views.metapolicy` for the master policy instead.

    **Required arguments:**

    ``domains``
        A list of domains from which to allow access. Each value may
        be either a domain name (e.g., ``example.com``) or a wildcard
        (e.g., ``*.example.com``). Due to serious potential security
        issues, it is strongly recommended that you not use wildcard
        domain values.

    **Optional arguments:**

    None.

    """
    return serve(request, policies.Policy(*domains))


def simple(request: HttpRequest, domains: Iterable[str]) -> HttpResponse:
    """
    Deprecated name for the ``allow_domains`` view.

    """
    warnings.warn(
        "flashpolicies.views.simple has been renamed to "
        "flashpolicies.views.allow_domains. Support for referring to it as "
        "flashpolicies.views.simple is deprecated and will be removed in a "
        "future release of django-flashpolicies.",
        DeprecationWarning,
    )
    return allow_domains(request, domains)


def metapolicy(
    request: HttpRequest, permitted: str, domains: Optional[Iterable[str]] = None
) -> HttpResponse:
    """
    Serves a cross-domain policy which can allow other policies
    to exist on the same domain.

    Note that this view, if used, must be the master policy for the
    domain, and so must be served from the URL ``/crossdomain.xml`` on
    the domain: setting metapolicy information in other policy files
    is forbidden by the cross-domain policy specification.

    **Required arguments:**

    ``permitted``
        A string indicating the extent to which other policies are
        permitted. A set of constants is available in
        ``flashpolicies.policies``, defining acceptable values for
        this argument.

    **Optional arguments:**

    ``domains``
        A list of domains from which to allow access. Each value may
        be either a domain name (e.g., ``example.com``) or a wildcard
        (e.g., ``*.example.com``). Due to serious potential security
        issues, it is strongly recommended that you not use wildcard
        domain values.

    """
    if domains is None:
        domains = []
    policy = policies.Policy(*domains)
    policy.metapolicy(permitted)
    return serve(request, policy)


def no_access(request: HttpRequest) -> HttpResponse:
    """
    Serves a cross-domain access policy which permits no access of any
    kind, via a metapolicy declaration disallowing all policy files.

    Note that this view, if used, must be the master policy for the
    domain, and so must be served from the URL ``/crossdomain.xml`` on
    the domain: setting metapolicy information in other policy files
    is forbidden by the cross-domain policy specification.

    **Required arguments:**

    None.

    **Optional arguments:**

    None.

    """
    return metapolicy(request, permitted=policies.SITE_CONTROL_NONE)



from django.test import SimpleTestCase

from flashpolicies import policies


class PolicyGeneratorTests(SimpleTestCase):
    """
    Tests the policy-file generation utilities.

    """

    dummy_fingerprint = "01:23:45:67:89:ab:cd:ef:01:23:45:67:89:ab:cd:ef:01:23:45:67"

    def test_policy_str(self):
        """
        Tests that str() on a Policy returns an object of type str with
        no encoding declared.

        """
        policy_str = str(policies.Policy())
        self.assertTrue(isinstance(policy_str, str))
        self.assertTrue(policy_str.startswith('<?xml version="1.0" ?>'))

    def test_policy_serialize(self):
        """
        Tests that serialize() returns an object of type bytes, with a
        declared encoding of UTF-8.

        """
        policy_bytes = policies.Policy().serialize()
        self.assertTrue(isinstance(policy_bytes, bytes))
        self.assertTrue(
            policy_bytes.startswith(b'<?xml version="1.0" encoding="utf-8"?>')
        )

    def test_policy_type(self):
        """
        Tests that the correct ``DOCTYPE`` declaration is generated.

        """
        policy = policies.Policy().xml_dom
        self.assertEqual(
            policy.doctype.systemId,
            "http://www.adobe.com/xml/dtds/cross-domain-policy.dtd",
        )
        self.assertEqual(policy.doctype.name, "cross-domain-policy")
        self.assertEqual(len(policy.childNodes), 2)

    def test_policy_root_element(self):
        """
        Tests that the correct root element is inserted.

        """
        policy = policies.Policy().xml_dom
        self.assertEqual(policy.documentElement.tagName, "cross-domain-policy")
        self.assertEqual(len(policy.documentElement.childNodes), 0)

    def test_allow_access_domain(self):
        """
        Tests that adding access for a domain inserts the proper
        element and attribute.

        """
        policy = policies.Policy()
        policy.allow_domain("media.example.com")
        xml_dom = policy.xml_dom
        self.assertEqual(len(xml_dom.documentElement.childNodes), 1)
        self.assertEqual(len(xml_dom.getElementsByTagName("allow-access-from")), 1)
        access_elem = xml_dom.getElementsByTagName("allow-access-from")[0]
        self.assertEqual(len(access_elem.attributes), 1)
        self.assertEqual(access_elem.getAttribute("domain"), "media.example.com")

    def test_allow_access_ports(self):
        """
        Tests that adding port access for socket connections inserts
        the proper attribute.

        """
        policy = policies.Policy()
        ports = ["80", "8080", "9000-1000"]
        policy.allow_domain("media.example.com", to_ports=ports)
        access_elem = policy.xml_dom.getElementsByTagName("allow-access-from")[0]
        self.assertEqual(len(access_elem.attributes), 2)
        self.assertEqual(access_elem.getAttribute("to-ports"), ",".join(ports))

    def test_allow_access_secure(self):
        """
        Tests that setting non-secure access for a domain inserts the
        proper attribute.

        """
        policy = policies.Policy()
        policy.allow_domain("media.example.com", secure=False)
        access_elem = policy.xml_dom.getElementsByTagName("allow-access-from")[0]
        self.assertEqual(len(access_elem.attributes), 2)
        self.assertEqual(access_elem.getAttribute("secure"), "false")

    def test_metapolicy(self):
        """
        Tests that adding metapolicy information inserts the proper
        element and attributes.

        """
        for permitted in policies.VALID_SITE_CONTROL:
            policy = policies.Policy()
            policy.metapolicy(permitted)
            xml_dom = policy.xml_dom
            self.assertEqual(len(xml_dom.documentElement.childNodes), 1)
            self.assertEqual(len(xml_dom.getElementsByTagName("site-control")), 1)
            control_elem = xml_dom.getElementsByTagName("site-control")[0]
            self.assertEqual(len(control_elem.attributes), 1)
            self.assertEqual(
                control_elem.getAttribute("permitted-cross-domain-policies"), permitted
            )

    def test_bad_metapolicy(self):
        """
        Tests that metapolicies are restricted to the values permitted
        by the specification.

        """
        policy = policies.Policy()
        self.assertRaises(TypeError, policy.metapolicy, "not-valid")

    def test_metapolicy_none_empty_domains(self):
        """
        Tests that setting the metapolicy to ``none`` clears the list
        of permitted domains.

        """
        policy = policies.Policy("media.example.com", "api.example.com")
        policy.metapolicy(policies.SITE_CONTROL_NONE)
        self.assertEqual(
            len(policy.xml_dom.getElementsByTagName("allow-access-from")), 0
        )

    def test_metapolicy_none_empty_identities(self):
        """
        Tests that setting the metapolicy to ``none`` clears the list
        of permitted cryptographic identities.

        """
        policy = policies.Policy()
        policy.allow_identity(self.dummy_fingerprint)
        policy.metapolicy(policies.SITE_CONTROL_NONE)
        self.assertEqual(
            len(policy.xml_dom.getElementsByTagName("allow-access-from-identity")), 0
        )

    def test_metapolicy_none_empty_headers(self):
        """
        Tests that setting the metapolicy to ``non`` clears the list of
        domains from which headers are permitted.

        """
        policy = policies.Policy()
        policy.allow_headers("media.example.com", ["SomeHeader"])
        policy.metapolicy(policies.SITE_CONTROL_NONE)
        self.assertEqual(
            len(policy.xml_dom.getElementsByTagName("allow-http-request-headers-from")),
            0,
        )

    def test_metapolicy_none_disallow_domains(self):
        """
        Tests that attempting to allow access from a domain, when the
        metapolicy is ``none``, raises an exception.

        """
        policy = policies.Policy()
        policy.metapolicy(policies.SITE_CONTROL_NONE)
        self.assertRaises(TypeError, policy.allow_domain, "media.example.com")

    def test_metapolicy_none_disallow_headers(self):
        """
        Tests that attempting to allow headers from a domain, when the
        metapolicy is ``none``, raises an exception.

        """
        policy = policies.Policy()
        policy.metapolicy(policies.SITE_CONTROL_NONE)
        self.assertRaises(
            TypeError, policy.allow_headers, "media.example.com", ["SomeHeader"]
        )

    def test_metapolicy_non_disallow_identities(self):
        """
        Tests that attempting to allow access from signed documents,
        when the metapolicy is ``none``, raises an exception.

        """
        policy = policies.Policy()
        policy.metapolicy(policies.SITE_CONTROL_NONE)
        self.assertRaises(TypeError, policy.allow_identity, self.dummy_fingerprint)

    def test_metapolicy_manual_tinkering(self):
        """
        Tests that setting metapolicy ``none``, then manually fiddling
        with the internals to try to allow things, will fail when the
        XML gets generated.

        """
        policy = policies.Policy()
        policy.metapolicy(policies.SITE_CONTROL_NONE)
        policy.domains["api.example.com"] = {"to_ports": None, "secure": True}
        with self.assertRaises(TypeError):
            policy.xml_dom

    def test_allow_http_headers(self):
        """
        Tests that allowing HTTP headers inserts the proper element and
        attributes.

        """
        domain = "media.example.com"
        headers = ["SomeHeader", "SomeOtherHeader"]
        policy = policies.Policy()
        policy.allow_headers(domain, headers, secure=False)
        xml_dom = policy.xml_dom
        self.assertEqual(
            len(xml_dom.getElementsByTagName("allow-http-request-headers-from")), 1
        )
        header_elem = xml_dom.getElementsByTagName("allow-http-request-headers-from")[0]
        self.assertEqual(len(header_elem.attributes), 3)
        self.assertEqual(header_elem.getAttribute("domain"), domain)
        self.assertEqual(header_elem.getAttribute("headers"), ",".join(headers))
        self.assertEqual(header_elem.getAttribute("secure"), "false")

    def test_allow_http_headers_secure(self):
        """
        Tests that setting non-secure access for HTTP headers from a
        domain inserts the proper attribute.

        """
        domain = "media.example.com"
        headers = ["SomeHeader", "SomeOtherHeader"]
        policy = policies.Policy()
        policy.allow_headers(domain, headers, secure=False)
        xml_dom = policy.xml_dom
        header_elem = xml_dom.getElementsByTagName("allow-http-request-headers-from")[0]
        self.assertEqual(len(header_elem.attributes), 3)
        self.assertEqual(header_elem.getAttribute("secure"), "false")

    def test_allow_identity(self):
        """
        Tests that allowing access from digitally-signed documents
        inserts the proper elements and attributes.

        """
        policy = policies.Policy()
        policy.allow_identity(self.dummy_fingerprint)
        xml_dom = policy.xml_dom
        identity_elem = xml_dom.getElementsByTagName("allow-access-from-identity")[0]
        self.assertEqual(len(identity_elem.childNodes), 1)
        signatory_elem = identity_elem.childNodes[0]
        self.assertEqual("signatory", signatory_elem.tagName)
        self.assertEqual(len(signatory_elem.childNodes), 1)
        certificate_elem = signatory_elem.childNodes[0]
        self.assertEqual("certificate", certificate_elem.tagName)
        self.assertEqual(len(certificate_elem.attributes), 2)
        self.assertEqual(
            "sha-1", certificate_elem.getAttribute("fingerprint-algorithm")
        )
        self.assertEqual(
            self.dummy_fingerprint, certificate_elem.getAttribute("fingerprint")
        )

    def test_simple_policy(self):
        """
        Tests that creating a simple policy with a list of domains
        returns a correct policy document.

        """
        domains = ["media.example.com", "api.example.com"]
        policy = policies.Policy(*domains)
        xml_dom = policy.xml_dom

        self.assertEqual(len(xml_dom.documentElement.childNodes), 2)
        self.assertEqual(len(xml_dom.getElementsByTagName("allow-access-from")), 2)

        domains_in_xml = [
            elem.getAttribute("domain")
            for elem in xml_dom.getElementsByTagName("allow-access-from")
        ]
        for domain in domains_in_xml:
            domains.remove(domain)



"""
These URLs are used by the test suite to exercise the various
views. You should not use these URLs in any sort of real deployment
situation.

"""

from django.urls import path

from flashpolicies import policies, views


def make_test_policy():
    policy = policies.Policy()
    policy.allow_domain("media.example.com")
    policy.allow_headers("media.example.com", ["SomeHeader"])
    return policy


urlpatterns = [
    path("crossdomain-serve.xml", views.serve, {"policy": make_test_policy()}),
    path(
        "crossdomain-allow-domains.xml",
        views.allow_domains,
        {"domains": ["media.example.com", "api.example.com"]},
    ),
    path(
        "crossdomain-simple-alias.xml",
        views.simple,
        {"domains": ["media.example.com", "api.example.com"]},
    ),
    path("crossdomain-no-access.xml", views.no_access),
    path(
        "crossdomain-metapolicy.xml",
        views.metapolicy,
        {"permitted": policies.SITE_CONTROL_ALL},
    ),
]



import os

from setuptools import find_packages, setup


setup(
    name="django-flashpolicies",
    zip_safe=False,  # eggs are the devil.
    version="1.12.1",
    description="Flash cross-domain policies for Django sites",
    long_description=open(os.path.join(os.path.dirname(__file__), "README.rst")).read(),
    author="James Bennett",
    author_email="james@b-list.org",
    url="https://github.com/ubernostrum/django-flashpolicies/",
    include_package_data=True,
    package_dir={"": "src"},
    packages=find_packages("src"),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
    ],
    python_requires=">=3.5",
    install_requires=["Django>=2.2"],
)
